@@ -5,7 +5,7 @@
 
 
 /*
- * Copyright (C) 2009-2016 Oracle Corporation
+ * Copyright (C) 2009-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -26,9 +26,9 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include <Protocol/ComponentName.h>
 #include <Protocol/ComponentName2.h>
 #include <Protocol/DriverBinding.h>
@@ -41,13 +41,13 @@
 #include <IndustryStandard/Pci22.h>
 
 #include "VBoxPkg.h"
-#include "iprt/asm.h"
 #include "VBoxVgaFonts.h"
+#include "iprt/asm.h"
 
 
-/*******************************************************************************
-*   Structures and Typedefs                                                    *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
 /**
  * Instance data for a VGA device this driver handles.
  */
@@ -71,9 +71,9 @@ typedef VBOXVGAMINIPORT *PVBOXVGAMINIPORT;
 #define VBOX_VGA_MINI_PORT_MAGIC_DEAD   0x19920406
 
 
-/*******************************************************************************
-*   Internal Functions                                                         *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Internal Functions                                                                                                           *
+*********************************************************************************************************************************/
 static EFI_STATUS EFIAPI
 VBoxVgaMiniPortDB_Supported(IN EFI_DRIVER_BINDING_PROTOCOL *This, IN EFI_HANDLE ControllerHandle,
                             IN EFI_DEVICE_PATH_PROTOCOL *RemainingDevicePath OPTIONAL);
@@ -109,9 +109,9 @@ VBoxVgaMiniPortCN2_GetControllerName(IN EFI_COMPONENT_NAME2_PROTOCOL *This,
                                      IN CHAR8 *Language,  OUT CHAR16 **ControllerName);
 
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 /** EFI Driver Binding Protocol. */
 static EFI_DRIVER_BINDING_PROTOCOL          g_VBoxVgaMiniPortDB =
 {
@@ -387,168 +387,155 @@ VBoxVgaMiniPortVMP_SetMode(IN EFI_VGA_MINI_PORT_PROTOCOL *This, IN UINTN ModeNum
 
     /* some initialization */
     ASMOutU8(0x3c2, 0xc3);
-    ASMOutU8(0x3c4, 0x04);
-    ASMOutU8(0x3c5, 0x02);
+    ASMOutU16(0x3c4, 0x0204);
 
-    /*
-     * inb(r63, 0x3da);                     // reset attr F/F
-     * outb(0x3c0, 0);                      // disable palette
-     * outb(0x3d4, 0x11); outb(0x3d5, 0);   // unprotect crtc regs 0 - 7
-     */
-    r[63] = ASMInU8((UINTN)0x3da);
-    ASMOutU8(0x3c0, 0);
-    ASMOutU16(0x3d4, 0x0011);
+    ASMInU8(0x3da);             // reset attr F/F
+    ASMOutU8(0x3c0, 0);         // disable palette
+    ASMOutU16(0x3d4, 0x0011);   // unprotect crtc regs 0 - 7
 
-#define BOUTB(count, aport, dport)  \
-     do {                                \
-         for (i = 0 ; i < count; ++i)    \
-         {                               \
-             ASMOutU8((aport), (UINT8)i);\
-             ASMOutU8((dport), r[i]);    \
-         }                               \
-     } while (0)
+#define BOUTB(count, aport, dport)                                  \
+    do {                                                            \
+        for (i = 0 ; i < (count); ++i)                              \
+            if ((dport) == (aport) + 1)                             \
+                ASMOutU16((aport), ((UINT16)r[i] << 8) | (UINT8)i); \
+            else {                                                  \
+                ASMOutU8((aport), (UINT8)i);                        \
+                ASMOutU8((dport), r[i]);                            \
+            }                                                       \
+    } while (0)
 
-    /*
-     *  Reset and set sequencer registers
-     *
-     * r0 = 0x01;  r1 = 0x00;  r2 = 0x03;  r3 = 0x00;  r4 = 0x02;
-     * boutb(5, 0x3c4, 0x3c5);
-     */
-     r[0] = 0x01;
-     r[1] = 0x00;
-     r[2] = 0x03;
-     r[3] = 0x00;
-     r[4] = 0x02;
-     BOUTB(5, 0x3c4, 0x3c5);
+    /* Reset and set sequencer registers */
+    r[0] = 0x01;
+    r[1] = 0x00;
+    r[2] = 0x03;
+    r[3] = 0x00;
+    r[4] = 0x02;
+    BOUTB(5, 0x3c4, 0x3c5);
 
-     /*
-      *  set misc out register
-      *
-      * outb(0x3c2, 0x67);
-      *
-      * r0 = 3
-      * boutb(1, 0x3c4, 0x3c5);         // enable sequencer
-      */
-     r[0] = 3;
-     BOUTB(1, 0x3c4, 0x3c5);
+    /* set misc out register */
+    ASMOutU8(0x3c2, 0x67);
 
-     /*  set all crtc registers */
-     r[0] = 0x5f;  r[1] = 0x4f;  r[2] = 0x50;  r[3] = 0x82;
-     r[4] = 0x55;  r[5] = 0x81;  r[6] = 0xbf;  r[7] = 0x1f;
-     r[8] = 0x00;  r[9] = 0x4f;  r[10]= 0x0d;  r[11]= 0x0e;
-     r[12]= 0x00;  r[13]= 0x00;  r[14]= 0x03;  r[15]= 0xc0;
-     r[16]= 0x9c;  r[17]= 0x0e;  r[18]= 0x8f;  r[19]= 0x28;
-     r[20]= 0x1f;  r[21]= 0x96;  r[22]= 0xb9;  r[23]= 0xa3;
-     r[24]= 0xff;
-     BOUTB(25, 0x3d4, 0x3d5);
+    /* enable sequencer */
+    ASMOutU16(0x3c4, 0x0300);
 
-     /*  set all graphics controller registers */
-     r[0]= 0x00;  r[1]= 0x00;  r[2]= 0x00;  r[3]= 0x00;
-     r[4]= 0x00;  r[5]= 0x10;  r[6]= 0x0e;  r[7]= 0x00;
-     r[8]= 0xff;
-     BOUTB(9, 0x3ce, 0x3cf);
+    /*  set all crtc registers */
+    r[0] = 0x5f;  r[1] = 0x4f;  r[2] = 0x50;  r[3] = 0x82;
+    r[4] = 0x55;  r[5] = 0x81;  r[6] = 0xbf;  r[7] = 0x1f;
+    r[8] = 0x00;  r[9] = 0x4f;  r[10]= 0x0d;  r[11]= 0x0e;
+    r[12]= 0x00;  r[13]= 0x00;  r[14]= 0x03;  r[15]= 0xc0;
+    r[16]= 0x9c;  r[17]= 0x0e;  r[18]= 0x8f;  r[19]= 0x28;
+    r[20]= 0x1f;  r[21]= 0x96;  r[22]= 0xb9;  r[23]= 0xa3;
+    r[24]= 0xff;
+    BOUTB(25, 0x3d4, 0x3d5);
 
-     /*  set all attribute registers */
-     r[63] = ASMInU8(0x3da);            // reset flip/flop
-     r[0] = 0x00;  r[1] = 0x01;  r[2] = 0x02;  r[3] = 0x03;
-     r[4] = 0x04;  r[5] = 0x05;  r[6] = 0x14;  r[7] = 0x07;
-     r[8] = 0x38;  r[9] = 0x39;  r[10]= 0x3a;  r[11]= 0x3b;
-     r[12]= 0x3c;  r[13]= 0x3d;  r[14]= 0x3e;  r[15]= 0x3f;
-     r[16]= 0x0c;  r[17]= 0x00;  r[18]= 0x0f;  r[19]= 0x08;
-     r[20]= 0x00;
-     BOUTB(21, 0x3c0, 0x3c0);
-     ASMOutU8(0x3c0, 0x20);             // re-enable palette
+    /*  set all graphics controller registers */
+    r[0]= 0x00;  r[1]= 0x00;  r[2]= 0x00;  r[3]= 0x00;
+    r[4]= 0x00;  r[5]= 0x10;  r[6]= 0x0e;  r[7]= 0x00;
+    r[8]= 0xff;
+    BOUTB(9, 0x3ce, 0x3cf);
 
-     /* set all VBox extended registers */
-     r[0] = 1;
-     BOUTB(1, 0x3c4, 0x3c5);            // disable sequencer
+    /*  set all attribute registers */
+    ASMInU8(0x3da);             // reset attr F/F
+    r[0] = 0x00;  r[1] = 0x01;  r[2] = 0x02;  r[3] = 0x03;
+    r[4] = 0x04;  r[5] = 0x05;  r[6] = 0x14;  r[7] = 0x07;
+    r[8] = 0x38;  r[9] = 0x39;  r[10]= 0x3a;  r[11]= 0x3b;
+    r[12]= 0x3c;  r[13]= 0x3d;  r[14]= 0x3e;  r[15]= 0x3f;
+    r[16]= 0x0c;  r[17]= 0x00;  r[18]= 0x0f;  r[19]= 0x08;
+    r[20]= 0x00;
+    BOUTB(21, 0x3c0, 0x3c0);
+    ASMOutU8(0x3c0, 0x20);              // re-enable palette
 
-     ASMOutU16(0x1ce, 0x04); ASMOutU16(0x1cf, 0);  // ENABLE
+    /* set all VBox extended registers */
 
-     r[0] = 3;
-     BOUTB(1, 0x3c4, 0x3c5);            // enable sequencer
+    /* enable sequencer */
+    ASMOutU16(0x3c4, 0x0100);
 
-     /* Load default values into the first 16 entries of the DAC */
-     {
-         static const UINT8 s_a3bVgaDac[64*3] =
-         {
-             0x00, 0x00, 0x00,
-             0x00, 0x00, 0x2A,
-             0x00, 0x2A, 0x00,
-             0x00, 0x2A, 0x2A,
-             0x2A, 0x00, 0x00,
-             0x2A, 0x00, 0x2A,
-             0x2A, 0x2A, 0x00,
-             0x2A, 0x2A, 0x2A,
-             0x00, 0x00, 0x15,
-             0x00, 0x00, 0x3F,
-             0x00, 0x2A, 0x15,
-             0x00, 0x2A, 0x3F,
-             0x2A, 0x00, 0x15,
-             0x2A, 0x00, 0x3F,
-             0x2A, 0x2A, 0x15,
-             0x2A, 0x2A, 0x3F,
-             0x00, 0x15, 0x00,
-             0x00, 0x15, 0x2A,
-             0x00, 0x3F, 0x00,
-             0x00, 0x3F, 0x2A,
-             0x2A, 0x15, 0x00,
-             0x2A, 0x15, 0x2A,
-             0x2A, 0x3F, 0x00,
-             0x2A, 0x3F, 0x2A,
-             0x00, 0x15, 0x15,
-             0x00, 0x15, 0x3F,
-             0x00, 0x3F, 0x15,
-             0x00, 0x3F, 0x3F,
-             0x2A, 0x15, 0x15,
-             0x2A, 0x15, 0x3F,
-             0x2A, 0x3F, 0x15,
-             0x2A, 0x3F, 0x3F,
-             0x15, 0x00, 0x00,
-             0x15, 0x00, 0x2A,
-             0x15, 0x2A, 0x00,
-             0x15, 0x2A, 0x2A,
-             0x3F, 0x00, 0x00,
-             0x3F, 0x00, 0x2A,
-             0x3F, 0x2A, 0x00,
-             0x3F, 0x2A, 0x2A,
-             0x15, 0x00, 0x15,
-             0x15, 0x00, 0x3F,
-             0x15, 0x2A, 0x15,
-             0x15, 0x2A, 0x3F,
-             0x3F, 0x00, 0x15,
-             0x3F, 0x00, 0x3F,
-             0x3F, 0x2A, 0x15,
-             0x3F, 0x2A, 0x3F,
-             0x15, 0x15, 0x00,
-             0x15, 0x15, 0x2A,
-             0x15, 0x3F, 0x00,
-             0x15, 0x3F, 0x2A,
-             0x3F, 0x15, 0x00,
-             0x3F, 0x15, 0x2A,
-             0x3F, 0x3F, 0x00,
-             0x3F, 0x3F, 0x2A,
-             0x15, 0x15, 0x15,
-             0x15, 0x15, 0x3F,
-             0x15, 0x3F, 0x15,
-             0x15, 0x3F, 0x3F,
-             0x3F, 0x15, 0x15,
-             0x3F, 0x15, 0x3F,
-             0x3F, 0x3F, 0x15,
-             0x3F, 0x3F, 0x3F
-          };
+    ASMOutU16(0x1ce, 0x04); ASMOutU16(0x1cf, 0);    // ENABLE
 
-          for (i = 0; i < 64; ++i)
-          {
-              ASMOutU8(0x3c8, (UINT8)i);
-              ASMOutU8(0x3c9, s_a3bVgaDac[i*3 + 0]);
-              ASMOutU8(0x3c9, s_a3bVgaDac[i*3 + 1]);
-              ASMOutU8(0x3c9, s_a3bVgaDac[i*3 + 2]);
-          }
-     }
+    /* enable sequencer */
+    ASMOutU16(0x3c4, 0x0300);
 
-     /* Load the appropriate font into the first map */
-     {
+    /* Load default values into the first 16 entries of the DAC */
+    {
+        static const UINT8 s_a3bVgaDac[64*3] =
+        {
+            0x00, 0x00, 0x00,
+            0x00, 0x00, 0x2A,
+            0x00, 0x2A, 0x00,
+            0x00, 0x2A, 0x2A,
+            0x2A, 0x00, 0x00,
+            0x2A, 0x00, 0x2A,
+            0x2A, 0x2A, 0x00,
+            0x2A, 0x2A, 0x2A,
+            0x00, 0x00, 0x15,
+            0x00, 0x00, 0x3F,
+            0x00, 0x2A, 0x15,
+            0x00, 0x2A, 0x3F,
+            0x2A, 0x00, 0x15,
+            0x2A, 0x00, 0x3F,
+            0x2A, 0x2A, 0x15,
+            0x2A, 0x2A, 0x3F,
+            0x00, 0x15, 0x00,
+            0x00, 0x15, 0x2A,
+            0x00, 0x3F, 0x00,
+            0x00, 0x3F, 0x2A,
+            0x2A, 0x15, 0x00,
+            0x2A, 0x15, 0x2A,
+            0x2A, 0x3F, 0x00,
+            0x2A, 0x3F, 0x2A,
+            0x00, 0x15, 0x15,
+            0x00, 0x15, 0x3F,
+            0x00, 0x3F, 0x15,
+            0x00, 0x3F, 0x3F,
+            0x2A, 0x15, 0x15,
+            0x2A, 0x15, 0x3F,
+            0x2A, 0x3F, 0x15,
+            0x2A, 0x3F, 0x3F,
+            0x15, 0x00, 0x00,
+            0x15, 0x00, 0x2A,
+            0x15, 0x2A, 0x00,
+            0x15, 0x2A, 0x2A,
+            0x3F, 0x00, 0x00,
+            0x3F, 0x00, 0x2A,
+            0x3F, 0x2A, 0x00,
+            0x3F, 0x2A, 0x2A,
+            0x15, 0x00, 0x15,
+            0x15, 0x00, 0x3F,
+            0x15, 0x2A, 0x15,
+            0x15, 0x2A, 0x3F,
+            0x3F, 0x00, 0x15,
+            0x3F, 0x00, 0x3F,
+            0x3F, 0x2A, 0x15,
+            0x3F, 0x2A, 0x3F,
+            0x15, 0x15, 0x00,
+            0x15, 0x15, 0x2A,
+            0x15, 0x3F, 0x00,
+            0x15, 0x3F, 0x2A,
+            0x3F, 0x15, 0x00,
+            0x3F, 0x15, 0x2A,
+            0x3F, 0x3F, 0x00,
+            0x3F, 0x3F, 0x2A,
+            0x15, 0x15, 0x15,
+            0x15, 0x15, 0x3F,
+            0x15, 0x3F, 0x15,
+            0x15, 0x3F, 0x3F,
+            0x3F, 0x15, 0x15,
+            0x3F, 0x15, 0x3F,
+            0x3F, 0x3F, 0x15,
+            0x3F, 0x3F, 0x3F
+        };
+
+        for (i = 0; i < 64; ++i)
+        {
+            ASMOutU8(0x3c8, (UINT8)i);
+            ASMOutU8(0x3c9, s_a3bVgaDac[i*3 + 0]);
+            ASMOutU8(0x3c9, s_a3bVgaDac[i*3 + 1]);
+            ASMOutU8(0x3c9, s_a3bVgaDac[i*3 + 2]);
+        }
+    }
+
+    /* Load the appropriate font into the first map */
+    {
         UINT8 const    *pabFont;
         unsigned        offBase = 0;
         UINT16          height;
