@@ -9,7 +9,7 @@
  */
 
 /*
- * Copyright (C) 2006-2016 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -72,8 +72,7 @@
 #include <VBox/vmm/ssm.h>
 
 #include "HGSMIHost.h"
-#include <VBox/HGSMI/HGSMIChannels.h>
-#include <VBox/HGSMI/HGSMIChSetup.h>
+#include <HGSMIChannels.h>
 
 #include "../DevVGASavedState.h"
 
