@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2016 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -37,6 +37,8 @@
 #include <iprt/asm.h>
 #include <iprt/buildconfig.h>
 #include <iprt/ctype.h>
+#include <iprt/file.h>
+#include <iprt/getopt.h>
 #include <iprt/initterm.h>
 #include <iprt/path.h>
 #include <iprt/stream.h>
@@ -119,6 +121,7 @@ static const VBMGCMD g_aCommands[] =
     { "modifyvm",           USAGE_MODIFYVM,         VBMG_CMD_TODO, handleModifyVM,             0 },
     { "startvm",            USAGE_STARTVM,          VBMG_CMD_TODO, handleStartVM,              0 },
     { "controlvm",          USAGE_CONTROLVM,        VBMG_CMD_TODO, handleControlVM,            0 },
+    { "unattended",         USAGE_UNATTENDEDINSTALL, HELP_CMD_UNATTENDED, handleUnattended,    0 },
     { "discardstate",       USAGE_DISCARDSTATE,     VBMG_CMD_TODO, handleDiscardState,         0 },
     { "adoptstate",         USAGE_ADOPTSTATE,       VBMG_CMD_TODO, handleAdoptState,           0 },
     { "snapshot",           USAGE_SNAPSHOT,         VBMG_CMD_TODO, handleSnapshot,             0 },
@@ -458,7 +461,9 @@ int main(int argc, char *argv[])
      * Before we do anything, init the runtime without loading
      * the support driver.
      */
-    RTR3InitExe(argc, &argv, 0);
+    int vrc = RTR3InitExe(argc, &argv, 0);
+    if (RT_FAILURE(vrc))
+        return RTMsgInitFailure(vrc);
 #if defined(RT_OS_WINDOWS) && !defined(VBOX_ONLY_DOCS)
     ATL::CComModule _Module; /* Required internally by ATL (constructor records instance in global variable). */
 #endif
@@ -472,6 +477,11 @@ int main(int argc, char *argv[])
     int  iCmdArg;
     const char *pszSettingsPw = NULL;
     const char *pszSettingsPwFile = NULL;
+#ifndef VBOX_ONLY_DOCS
+    int         cResponseFileArgs     = 0;
+    char      **papszResponseFileArgs = NULL;
+    char      **papszNewArgv          = NULL;
+#endif
 
     for (int i = 1; i < argc || argc <= iCmd; i++)
     {
@@ -546,6 +556,47 @@ int main(int argc, char *argv[])
             pszSettingsPwFile = argv[i+1];
             iCmd += 2;
         }
+#ifndef VBOX_ONLY_DOCS
+        else if (argv[i][0] == '@')
+        {
+            if (papszResponseFileArgs)
+                return RTMsgErrorExitFailure("Only one response file allowed");
+
+            /* Load response file, making sure it's valid UTF-8. */
+            char  *pszResponseFile;
+            size_t cbResponseFile;
+            vrc = RTFileReadAllEx(&argv[i][1], 0, RTFOFF_MAX, RTFILE_RDALL_O_DENY_NONE | RTFILE_RDALL_F_TRAILING_ZERO_BYTE,
+                                  (void **)&pszResponseFile, &cbResponseFile);
+            if (RT_FAILURE(vrc))
+                return RTMsgErrorExitFailure("Error reading response file '%s': %Rrc", &argv[i][1], vrc);
+            vrc = RTStrValidateEncoding(pszResponseFile);
+            if (RT_FAILURE(vrc))
+            {
+                RTFileReadAllFree(pszResponseFile, cbResponseFile);
+                return RTMsgErrorExitFailure("Invalid response file ('%s') encoding: %Rrc", &argv[i][1], vrc);
+            }
+
+            /* Parse it. */
+            vrc = RTGetOptArgvFromString(&papszResponseFileArgs, &cResponseFileArgs, pszResponseFile,
+                                         RTGETOPTARGV_CNV_QUOTE_BOURNE_SH, NULL);
+            RTFileReadAllFree(pszResponseFile, cbResponseFile);
+            if (RT_FAILURE(vrc))
+                return RTMsgErrorExitFailure("Failed to parse response file '%s' (bourne shell style): %Rrc", &argv[i][1], vrc);
+
+            /* Construct new argv+argc with the response file arguments inserted. */
+            int cNewArgs = argc + cResponseFileArgs;
+            papszNewArgv = (char **)RTMemAllocZ((cNewArgs + 2) * sizeof(papszNewArgv[0]));
+            if (!papszNewArgv)
+                return RTMsgErrorExitFailure("out of memory");
+            memcpy(&papszNewArgv[0], &argv[0], sizeof(argv[0]) * (i + 1));
+            memcpy(&papszNewArgv[i + 1], papszResponseFileArgs, sizeof(argv[0]) * cResponseFileArgs);
+            memcpy(&papszNewArgv[i + 1 + cResponseFileArgs], &argv[i + 1], sizeof(argv[0]) * (argc - i - 1 + 1));
+            argv = papszNewArgv;
+            argc = argc + cResponseFileArgs;
+
+            iCmd++;
+        }
+#endif
         else
             break;
     }
@@ -695,6 +746,12 @@ int main(int argc, char *argv[])
         rcExit = pCmd->pfnHandler(&handlerArg);
     }
 
+    if (papszResponseFileArgs)
+    {
+        RTGetOptArgvFree(papszResponseFileArgs);
+        RTMemFree(papszNewArgv);
+    }
+
     return rcExit;
 #else  /* VBOX_ONLY_DOCS */
     return RTEXITCODE_SUCCESS;
