@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2015-2016 Oracle Corporation
+ * Copyright (C) 2015-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,12 +22,10 @@
 /* Qt includes: */
 # include <QApplication>
 # include <QDesktopWidget>
+# include <QScreen>
 # ifdef VBOX_WS_X11
 #  include <QTimer>
 # endif
-# if QT_VERSION >= 0x050000
-#  include <QScreen>
-# endif /* QT_VERSION >= 0x050000 */
 
 /* GUI includes: */
 # include "UIDesktopWidgetWatchdog.h"
@@ -342,7 +340,7 @@ const QRegion UIDesktopWidgetWatchdog::overallAvailableRegion() const
     return region;
 }
 
-#if defined(VBOX_WS_X11) && QT_VERSION >= 0x050000
+#ifdef VBOX_WS_X11
 bool UIDesktopWidgetWatchdog::isFakeScreenDetected() const
 {
     // WORKAROUND:
@@ -354,60 +352,34 @@ bool UIDesktopWidgetWatchdog::isFakeScreenDetected() const
     return    qApp->screens().size() == 0 /* zero-screen case is impossible after 5.6.1 */
            || (qApp->screens().size() == 1 && qApp->screens().first()->name() == ":0.0");
 }
-#endif /* VBOX_WS_X11 && QT_VERSION >= 0x050000 */
+#endif /* VBOX_WS_X11 */
 
-#if QT_VERSION < 0x050000
-
-void UIDesktopWidgetWatchdog::sltHandleHostScreenCountChanged(int cHostScreenCount)
+double UIDesktopWidgetWatchdog::devicePixelRatio(int iHostScreenIndex)
 {
-//    printf("UIDesktopWidgetWatchdog::sltHandleHostScreenCountChanged(%d)\n", cHostScreenCount);
-
-# ifdef VBOX_WS_X11
-    /* Update host-screen configuration: */
-    updateHostScreenConfiguration(cHostScreenCount);
-# endif /* VBOX_WS_X11 */
-
-    /* Notify listeners: */
-    emit sigHostScreenCountChanged(cHostScreenCount);
+    /* First, we should check whether the screen is valid: */
+    QScreen *pScreen = iHostScreenIndex == -1
+                     ? QGuiApplication::primaryScreen()
+                     : QGuiApplication::screens().value(iHostScreenIndex);
+    AssertPtrReturn(pScreen, 1.0);
+    /* Then acquire device-pixel-ratio: */
+    return pScreen->devicePixelRatio();
 }
 
-void UIDesktopWidgetWatchdog::sltHandleHostScreenResized(int iHostScreenIndex)
+double UIDesktopWidgetWatchdog::devicePixelRatio(QWidget *pWidget)
 {
-//    printf("UIDesktopWidgetWatchdog::sltHandleHostScreenResized(%d)\n", iHostScreenIndex);
-
-# ifdef VBOX_WS_X11
-    /* Update host-screen available-geometry: */
-    updateHostScreenAvailableGeometry(iHostScreenIndex);
-# endif /* VBOX_WS_X11 */
-
-    /* Notify listeners: */
-    emit sigHostScreenResized(iHostScreenIndex);
+    /* Redirect call to wrapper above: */
+    return devicePixelRatio(screenNumber(pWidget));
 }
 
-void UIDesktopWidgetWatchdog::sltHandleHostScreenWorkAreaResized(int iHostScreenIndex)
-{
-//    printf("UIDesktopWidgetWatchdog::sltHandleHostScreenWorkAreaResized(%d)\n", iHostScreenIndex);
-
-# ifdef VBOX_WS_X11
-    /* Update host-screen available-geometry: */
-    updateHostScreenAvailableGeometry(iHostScreenIndex);
-# endif /* VBOX_WS_X11 */
-
-    /* Notify listeners: */
-    emit sigHostScreenWorkAreaResized(iHostScreenIndex);
-}
-
-#else /* QT_VERSION >= 0x050000 */
-
 void UIDesktopWidgetWatchdog::sltHostScreenAdded(QScreen *pHostScreen)
 {
 //    printf("UIDesktopWidgetWatchdog::sltHostScreenAdded(%d)\n", screenCount());
 
     /* Listen for screen signals: */
-    connect(pHostScreen, SIGNAL(geometryChanged(const QRect &)),
-            this, SLOT(sltHandleHostScreenResized(const QRect &)));
-    connect(pHostScreen, SIGNAL(availableGeometryChanged(const QRect &)),
-            this, SLOT(sltHandleHostScreenWorkAreaResized(const QRect &)));
+    connect(pHostScreen, &QScreen::geometryChanged,
+            this, &UIDesktopWidgetWatchdog::sltHandleHostScreenResized);
+    connect(pHostScreen, &QScreen::availableGeometryChanged,
+            this, &UIDesktopWidgetWatchdog::sltHandleHostScreenWorkAreaResized);
 
 # ifdef VBOX_WS_X11
     /* Update host-screen configuration: */
@@ -423,10 +395,10 @@ void UIDesktopWidgetWatchdog::sltHostScreenRemoved(QScreen *pHostScreen)
 //    printf("UIDesktopWidgetWatchdog::sltHostScreenRemoved(%d)\n", screenCount());
 
     /* Forget about screen signals: */
-    disconnect(pHostScreen, SIGNAL(geometryChanged(const QRect &)),
-               this, SLOT(sltHandleHostScreenResized(const QRect &)));
-    disconnect(pHostScreen, SIGNAL(availableGeometryChanged(const QRect &)),
-               this, SLOT(sltHandleHostScreenWorkAreaResized(const QRect &)));
+    disconnect(pHostScreen, &QScreen::geometryChanged,
+               this, &UIDesktopWidgetWatchdog::sltHandleHostScreenResized);
+    disconnect(pHostScreen, &QScreen::availableGeometryChanged,
+               this, &UIDesktopWidgetWatchdog::sltHandleHostScreenWorkAreaResized);
 
 # ifdef VBOX_WS_X11
     /* Update host-screen configuration: */
@@ -483,8 +455,6 @@ void UIDesktopWidgetWatchdog::sltHandleHostScreenWorkAreaResized(const QRect &av
     emit sigHostScreenWorkAreaResized(iHostScreenIndex);
 }
 
-#endif /* QT_VERSION >= 0x050000 */
-
 #ifdef VBOX_WS_X11
 void UIDesktopWidgetWatchdog::sltHandleHostScreenAvailableGeometryCalculated(int iHostScreenIndex, QRect availableGeometry)
 {
@@ -511,21 +481,17 @@ void UIDesktopWidgetWatchdog::sltHandleHostScreenAvailableGeometryCalculated(int
 void UIDesktopWidgetWatchdog::prepare()
 {
     /* Prepare connections: */
-#if QT_VERSION < 0x050000
-    connect(QApplication::desktop(), SIGNAL(screenCountChanged(int)), this, SLOT(sltHandleHostScreenCountChanged(int)));
-    connect(QApplication::desktop(), SIGNAL(resized(int)), this, SLOT(sltHandleHostScreenResized(int)));
-    connect(QApplication::desktop(), SIGNAL(workAreaResized(int)), this, SLOT(sltHandleHostScreenWorkAreaResized(int)));
-#else /* QT_VERSION >= 0x050000 */
-    connect(qApp, SIGNAL(screenAdded(QScreen *)), this, SLOT(sltHostScreenAdded(QScreen *)));
-    connect(qApp, SIGNAL(screenRemoved(QScreen *)), this, SLOT(sltHostScreenRemoved(QScreen *)));
+    connect(qApp, &QGuiApplication::screenAdded,
+            this, &UIDesktopWidgetWatchdog::sltHostScreenAdded);
+    connect(qApp, &QGuiApplication::screenRemoved,
+            this, &UIDesktopWidgetWatchdog::sltHostScreenRemoved);
     foreach (QScreen *pHostScreen, qApp->screens())
     {
-        connect(pHostScreen, SIGNAL(geometryChanged(const QRect &)),
-                this, SLOT(sltHandleHostScreenResized(const QRect &)));
-        connect(pHostScreen, SIGNAL(availableGeometryChanged(const QRect &)),
-                this, SLOT(sltHandleHostScreenWorkAreaResized(const QRect &)));
+        connect(pHostScreen, &QScreen::geometryChanged,
+                this, &UIDesktopWidgetWatchdog::sltHandleHostScreenResized);
+        connect(pHostScreen, &QScreen::availableGeometryChanged,
+                this, &UIDesktopWidgetWatchdog::sltHandleHostScreenWorkAreaResized);
     }
-#endif /* QT_VERSION >= 0x050000 */
 
 #ifdef VBOX_WS_X11
     /* Update host-screen configuration: */
@@ -536,21 +502,17 @@ void UIDesktopWidgetWatchdog::prepare()
 void UIDesktopWidgetWatchdog::cleanup()
 {
     /* Cleanup connections: */
-#if QT_VERSION < 0x050000
-    disconnect(QApplication::desktop(), SIGNAL(screenCountChanged(int)), this, SLOT(sltHandleHostScreenCountChanged(int)));
-    disconnect(QApplication::desktop(), SIGNAL(resized(int)), this, SLOT(sltHandleHostScreenResized(int)));
-    disconnect(QApplication::desktop(), SIGNAL(workAreaResized(int)), this, SLOT(sltHandleHostScreenWorkAreaResized(int)));
-#else /* QT_VERSION >= 0x050000 */
-    disconnect(qApp, SIGNAL(screenAdded(QScreen *)), this, SLOT(sltHostScreenAdded(QScreen *)));
-    disconnect(qApp, SIGNAL(screenRemoved(QScreen *)), this, SLOT(sltHostScreenRemoved(QScreen *)));
+    disconnect(qApp, &QGuiApplication::screenAdded,
+               this, &UIDesktopWidgetWatchdog::sltHostScreenAdded);
+    disconnect(qApp, &QGuiApplication::screenRemoved,
+               this, &UIDesktopWidgetWatchdog::sltHostScreenRemoved);
     foreach (QScreen *pHostScreen, qApp->screens())
     {
-        disconnect(pHostScreen, SIGNAL(geometryChanged(const QRect &)),
-                   this, SLOT(sltHandleHostScreenResized(const QRect &)));
-        disconnect(pHostScreen, SIGNAL(availableGeometryChanged(const QRect &)),
-                   this, SLOT(sltHandleHostScreenWorkAreaResized(const QRect &)));
+        disconnect(pHostScreen, &QScreen::geometryChanged,
+                   this, &UIDesktopWidgetWatchdog::sltHandleHostScreenResized);
+        disconnect(pHostScreen, &QScreen::availableGeometryChanged,
+                   this, &UIDesktopWidgetWatchdog::sltHandleHostScreenWorkAreaResized);
     }
-#endif /* QT_VERSION >= 0x050000 */
 
 #ifdef VBOX_WS_X11
     /* Cleanup existing workers finally: */
@@ -597,8 +559,8 @@ void UIDesktopWidgetWatchdog::updateHostScreenAvailableGeometry(int iHostScreenI
         const QRect hostScreenGeometry = screenGeometry(iHostScreenIndex);
 
         /* Connect worker listener: */
-        connect(pWorker, SIGNAL(sigHostScreenAvailableGeometryCalculated(int, QRect)),
-                this, SLOT(sltHandleHostScreenAvailableGeometryCalculated(int, QRect)));
+        connect(pWorker, &UIInvisibleWindow::sigHostScreenAvailableGeometryCalculated,
+                this, &UIDesktopWidgetWatchdog::sltHandleHostScreenAvailableGeometryCalculated);
 
         /* Place worker to corresponding host-screen: */
         pWorker->move(hostScreenGeometry.center());
