@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2014-2016 Oracle Corporation
+ * Copyright (C) 2014-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -20,6 +20,7 @@
 #else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
 
 /* Qt includes: */
+# include <QAccessibleWidget>
 # include <QStylePainter>
 # include <QStyleOption>
 # include <QHBoxLayout>
@@ -72,6 +73,8 @@ public:
     /** Returns button size-hint. */
     QSize sizeHint() const { return m_size; }
 
+    /** Returns whether button is checked. */
+    bool isChecked() const;
     /** Defines whether button is @a fChecked. */
     void setChecked(bool fChecked);
 
@@ -111,6 +114,82 @@ private:
 };
 
 
+/** QAccessibleWidget extension used as an accessibility interface for UIStatusBarEditor buttons. */
+class UIAccessibilityInterfaceForUIStatusBarEditorButton : public QAccessibleWidget
+{
+public:
+
+    /** Returns an accessibility interface for passed @a strClassname and @a pObject. */
+    static QAccessibleInterface *pFactory(const QString &strClassname, QObject *pObject);
+
+    /** Constructs an accessibility interface passing @a pWidget to the base-class. */
+    UIAccessibilityInterfaceForUIStatusBarEditorButton(QWidget *pWidget);
+
+    /** Returns a text for the passed @a enmTextRole. */
+    virtual QString text(QAccessible::Text enmTextRole) const /* override */;
+
+    /** Returns the state. */
+    virtual QAccessible::State state() const /* override */;
+
+private:
+
+    /** Returns corresponding toolbar button. */
+    UIStatusBarEditorButton *button() const { return qobject_cast<UIStatusBarEditorButton*>(widget()); }
+};
+
+
+/*********************************************************************************************************************************
+*   Class UIAccessibilityInterfaceForUIStatusBarEditorButton implementation.                                                     *
+*********************************************************************************************************************************/
+
+/* static */
+QAccessibleInterface *UIAccessibilityInterfaceForUIStatusBarEditorButton::pFactory(const QString &strClassname, QObject *pObject)
+{
+    /* Creating toolbar button accessibility interface: */
+    if (   pObject
+        && strClassname == QLatin1String("UIStatusBarEditorButton"))
+        return new UIAccessibilityInterfaceForUIStatusBarEditorButton(qobject_cast<QWidget*>(pObject));
+
+    /* Null by default: */
+    return 0;
+}
+
+UIAccessibilityInterfaceForUIStatusBarEditorButton::UIAccessibilityInterfaceForUIStatusBarEditorButton(QWidget *pWidget)
+    : QAccessibleWidget(pWidget, QAccessible::CheckBox)
+{
+}
+
+QString UIAccessibilityInterfaceForUIStatusBarEditorButton::text(QAccessible::Text /* enmTextRole */) const
+{
+    /* Make sure view still alive: */
+    AssertPtrReturn(button(), QString());
+
+    /* Return view tool-tip: */
+    return gpConverter->toString(button()->type());
+}
+
+QAccessible::State UIAccessibilityInterfaceForUIStatusBarEditorButton::state() const /* override */
+{
+    /* Prepare the button state: */
+    QAccessible::State state;
+
+    /* Make sure button still alive: */
+    AssertPtrReturn(button(), state);
+
+    /* Compose the button state: */
+    state.checkable = true;
+    state.checked = button()->isChecked();
+
+    /* Return the button state: */
+    return state;
+}
+
+
+
+/*********************************************************************************************************************************
+*   Class UIStatusBarEditorButton implementation.                                                                                *
+*********************************************************************************************************************************/
+
 /* static */
 const QString UIStatusBarEditorButton::MimeType = QString("application/virtualbox;value=IndicatorType");
 
@@ -139,6 +218,11 @@ UIStatusBarEditorButton::UIStatusBarEditorButton(IndicatorType type)
     retranslateUi();
 }
 
+bool UIStatusBarEditorButton::isChecked() const
+{
+    return m_fChecked;
+}
+
 void UIStatusBarEditorButton::setChecked(bool fChecked)
 {
     /* Update 'checked' state: */
@@ -395,6 +479,9 @@ void UIStatusBarEditorWidget::prepare()
     if (m_strMachineID.isEmpty())
         return;
 
+    /* Install tool-bar button accessibility interface factory: */
+    QAccessible::installFactory(UIAccessibilityInterfaceForUIStatusBarEditorButton::pFactory);
+
     /* Track D&D events: */
     setAcceptDrops(true);
 
@@ -405,17 +492,20 @@ void UIStatusBarEditorWidget::prepare()
         /* Configure main-layout: */
         int iLeft, iTop, iRight, iBottom;
         m_pMainLayout->getContentsMargins(&iLeft, &iTop, &iRight, &iBottom);
-        /* Standard margins should not be too big: */
-        iLeft   = qMin(iLeft,   10);
-        iTop    = qMin(iTop,    10);
-        iRight  = qMin(iRight,  10);
-        iBottom = qMin(iBottom, 10);
+        /* Acquire metric: */
+        const int iStandardMetric = QApplication::style()->pixelMetric(QStyle::PM_SmallIconSize) / 2;
+        const int iMinimumMetric = QApplication::style()->pixelMetric(QStyle::PM_SmallIconSize) / 4;
+        /* Standard margins should not be too small/large: */
+        iLeft   = iStandardMetric;
+        iTop    = iStandardMetric;
+        iRight  = iStandardMetric;
+        iBottom = iStandardMetric;
         /* Bottom margin should be smaller for the common case: */
-        if (iBottom >= 5)
-            iBottom -= 5;
+        if (iBottom >= iMinimumMetric)
+            iBottom -= iMinimumMetric;
         /* Left margin should be bigger for the settings case: */
         if (m_fStartedFromVMSettings)
-            iLeft += 5;
+            iLeft += iMinimumMetric;
         /* Apply margins/spacing finally: */
         m_pMainLayout->setContentsMargins(iLeft, iTop, iRight, iBottom);
         m_pMainLayout->setSpacing(0);
@@ -454,7 +544,11 @@ void UIStatusBarEditorWidget::prepare()
         {
             /* Configure button-layout: */
             m_pButtonLayout->setContentsMargins(0, 0, 0, 0);
+#ifdef VBOX_WS_MAC
             m_pButtonLayout->setSpacing(5);
+#else
+            m_pButtonLayout->setSpacing(qApp->style()->pixelMetric(QStyle::PM_LayoutHorizontalSpacing) / 2);
+#endif
             /* Add button-layout into main-layout: */
             m_pMainLayout->addLayout(m_pButtonLayout);
         }
@@ -535,53 +629,60 @@ void UIStatusBarEditorWidget::paintEvent(QPaintEvent*)
     QColor color3 = pal.color(QPalette::Window).darker(120);
 #endif /* VBOX_WS_WIN || VBOX_WS_X11 */
 
+    /* Acquire metric: */
+    const int iMetric = QApplication::style()->pixelMetric(QStyle::PM_SmallIconSize) / 4;
+
     /* Left corner: */
-    QRadialGradient grad1(QPointF(5, 5), 5);
+    QRadialGradient grad1(QPointF(iMetric, iMetric), iMetric);
     {
         grad1.setColorAt(0, color2);
         grad1.setColorAt(1, color1);
     }
     /* Right corner: */
-    QRadialGradient grad2(QPointF(width() - 5, 5), 5);
+    QRadialGradient grad2(QPointF(width() - iMetric, iMetric), iMetric);
     {
         grad2.setColorAt(0, color2);
         grad2.setColorAt(1, color1);
     }
     /* Top line: */
-    QLinearGradient grad3(QPointF(5, 0), QPointF(5, 5));
+    QLinearGradient grad3(QPointF(iMetric, 0), QPointF(iMetric, iMetric));
     {
         grad3.setColorAt(0, color1);
         grad3.setColorAt(1, color2);
     }
     /* Left line: */
-    QLinearGradient grad4(QPointF(0, 5), QPointF(5, 5));
+    QLinearGradient grad4(QPointF(0, iMetric), QPointF(iMetric, iMetric));
     {
         grad4.setColorAt(0, color1);
         grad4.setColorAt(1, color2);
     }
     /* Right line: */
-    QLinearGradient grad5(QPointF(width(), 5), QPointF(width() - 5, 5));
+    QLinearGradient grad5(QPointF(width(), iMetric), QPointF(width() - iMetric, iMetric));
     {
         grad5.setColorAt(0, color1);
         grad5.setColorAt(1, color2);
     }
 
     /* Paint shape/shadow: */
-    painter.fillRect(QRect(5, 5, width() - 5 * 2, height() - 5), color0); // background
-    painter.fillRect(QRect(0,           0, 5, 5), grad1); // left corner
-    painter.fillRect(QRect(width() - 5, 0, 5, 5), grad2); // right corner
-    painter.fillRect(QRect(5, 0, width() - 5 * 2, 5), grad3); // bottom line
-    painter.fillRect(QRect(0,           5, 5, height() - 5), grad4); // left line
-    painter.fillRect(QRect(width() - 5, 5, 5, height() - 5), grad5); // right line
+    painter.fillRect(QRect(iMetric, iMetric, width() - iMetric * 2, height() - iMetric), color0); // background
+    painter.fillRect(QRect(0,                 0, iMetric, iMetric), grad1); // left corner
+    painter.fillRect(QRect(width() - iMetric, 0, iMetric, iMetric), grad2); // right corner
+    painter.fillRect(QRect(iMetric,           0, width() - iMetric * 2, iMetric), grad3); // bottom line
+    painter.fillRect(QRect(0,                 iMetric, iMetric, height() - iMetric), grad4); // left line
+    painter.fillRect(QRect(width() - iMetric, iMetric, iMetric, height() - iMetric), grad5); // right line
 
 #if defined(VBOX_WS_WIN) || defined(VBOX_WS_X11)
     /* Paint frames: */
     painter.save();
     painter.setPen(color3);
-    painter.drawLine(QLine(QPoint(5 + 1, 5 + 1),                      QPoint(width() - 1 - 5 - 1, 5 + 1)));
-    painter.drawLine(QLine(QPoint(width() - 1 - 5 - 1, 5 + 1),        QPoint(width() - 1 - 5 - 1, height() - 1)));
-    painter.drawLine(QLine(QPoint(width() - 1 - 5 - 1, height() - 1), QPoint(5 + 1, height() - 1)));
-    painter.drawLine(QLine(QPoint(5 + 1, height() - 1),               QPoint(5 + 1, 5 + 1)));
+    painter.drawLine(QLine(QPoint(iMetric + 1,               iMetric + 1),
+                           QPoint(width() - 1 - iMetric - 1, iMetric + 1)));
+    painter.drawLine(QLine(QPoint(width() - 1 - iMetric - 1, iMetric + 1),
+                           QPoint(width() - 1 - iMetric - 1, height() - 1)));
+    painter.drawLine(QLine(QPoint(width() - 1 - iMetric - 1, height() - 1),
+                           QPoint(iMetric + 1,               height() - 1)));
+    painter.drawLine(QLine(QPoint(iMetric + 1,               height() - 1),
+                           QPoint(iMetric + 1,               iMetric + 1)));
     painter.restore();
 #endif /* VBOX_WS_WIN || VBOX_WS_X11 */
 
@@ -592,10 +693,10 @@ void UIStatusBarEditorWidget::paintEvent(QPaintEvent*)
         option.state |= QStyle::State_Horizontal;
         const QRect geo = m_pButtonDropToken->geometry();
         option.rect = !m_fDropAfterTokenButton ?
-                      QRect(geo.topLeft() - QPoint(5, 5),
-                            geo.bottomLeft() + QPoint(0, 5)) :
-                      QRect(geo.topRight() - QPoint(0, 5),
-                            geo.bottomRight() + QPoint(5, 5));
+                      QRect(geo.topLeft() - QPoint(iMetric, iMetric),
+                            geo.bottomLeft() + QPoint(0, iMetric)) :
+                      QRect(geo.topRight() - QPoint(0, iMetric),
+                            geo.bottomRight() + QPoint(iMetric, iMetric));
         QApplication::style()->drawPrimitive(QStyle::PE_IndicatorToolBarSeparator,
                                              &option, &painter);
     }
