@@ -134,29 +134,28 @@ FNIEMOP_STUB(iemOp_vpmuldq_Vx_Hx_Wx);
 /** Opcode VEX.66.0F38 0x29. */
 FNIEMOP_STUB(iemOp_vpcmpeqq_Vx_Hx_Wx);
 
-/** Opcode VEX.66.0F38 0x2a. */
-/*
- * @ opcode      0x2a
- * @ opcodesub   !11 mr/reg vex.l=0
- * @ oppfx       0x66
- * @ opcpuid     avx
- * @ opgroup     og_avx_cachect
- * @ opxcpttype  1
- * @ optest      op1=-1 op2=2  -> op1=2
- * @ optest      op1=0 op2=-42 -> op1=-42
- */
+
 FNIEMOP_DEF(iemOp_vmovntdqa_Vx_Mx)
 {
     Assert(pVCpu->iem.s.uVexLength <= 1);
-    IEMOP_MNEMONIC2(VEX_RM, VMOVNTDQA, vmovntdqa, Vx_WO, Mx, DISOPTYPE_HARMLESS, IEMOPHINT_IGNORES_OP_SIZES);
     uint8_t bRm; IEM_OPCODE_GET_NEXT_U8(&bRm);
     if ((bRm & X86_MODRM_MOD_MASK) != (3 << X86_MODRM_MOD_SHIFT))
     {
         if (pVCpu->iem.s.uVexLength == 0)
         {
-            /*
-             * 128-bit: Memory, register.
+            /**
+             * @opcode      0x2a
+             * @opcodesub   !11 mr/reg vex.l=0
+             * @oppfx       0x66
+             * @opcpuid     avx
+             * @opgroup     og_avx_cachect
+             * @opxcpttype  1
+             * @optest      op1=-1 op2=2  -> op1=2
+             * @optest      op1=0 op2=-42 -> op1=-42
              */
+            /* 128-bit: Memory, register. */
+            IEMOP_MNEMONIC2EX(vmovntdqa_Vdq_WO_Mdq_L0, "vmovntdqa, Vdq_WO, Mdq", VEX_RM_MEM, VMOVNTDQA, vmovntdqa, Vx_WO, Mx,
+                              DISOPTYPE_HARMLESS, IEMOPHINT_IGNORES_OP_SIZES);
             IEM_MC_BEGIN(0, 2);
             IEM_MC_LOCAL(RTUINT128U,                uSrc);
             IEM_MC_LOCAL(RTGCPTR,                   GCPtrEffSrc);
@@ -174,20 +173,20 @@ FNIEMOP_DEF(iemOp_vmovntdqa_Vx_Mx)
         }
         else
         {
-/*
- * @ opdone
- * @ opcode      0x2a
- * @ opcodesub   !11 mr/reg vex.l=1
- * @ oppfx       0x66
- * @ opcpuid     avx2
- * @ opgroup     og_avx2_cachect
- * @ opxcpttype  1
- * @ optest      op1=-1 op2=2  -> op1=2
- * @ optest      op1=0 op2=-42 -> op1=-42
- */
-            /*
-             * 256-bit: Memory, register.
+            /**
+             * @opdone
+             * @opcode      0x2a
+             * @opcodesub   !11 mr/reg vex.l=1
+             * @oppfx       0x66
+             * @opcpuid     avx2
+             * @opgroup     og_avx2_cachect
+             * @opxcpttype  1
+             * @optest      op1=-1 op2=2  -> op1=2
+             * @optest      op1=0 op2=-42 -> op1=-42
              */
+            /* 256-bit: Memory, register. */
+            IEMOP_MNEMONIC2EX(vmovntdqa_Vqq_WO_Mqq_L1, "vmovntdqa, Vqq_WO,Mqq", VEX_RM_MEM, VMOVNTDQA, vmovntdqa, Vx_WO, Mx,
+                              DISOPTYPE_HARMLESS, IEMOPHINT_IGNORES_OP_SIZES);
             IEM_MC_BEGIN(0, 2);
             IEM_MC_LOCAL(RTUINT256U,                uSrc);
             IEM_MC_LOCAL(RTGCPTR,                   GCPtrEffSrc);
@@ -195,31 +194,32 @@ FNIEMOP_DEF(iemOp_vmovntdqa_Vx_Mx)
             IEM_MC_CALC_RM_EFF_ADDR(GCPtrEffSrc, bRm, 0);
             IEMOP_HLP_DONE_VEX_DECODING_NO_VVVV();
             IEM_MC_MAYBE_RAISE_AVX2_RELATED_XCPT();
-            IEM_MC_ACTUALIZE_AVX_STATE_FOR_READ();
+            IEM_MC_ACTUALIZE_AVX_STATE_FOR_CHANGE();
 
-            IEM_MC_FETCH_YREG_U256(uSrc, ((bRm >> X86_MODRM_REG_SHIFT) & X86_MODRM_REG_SMASK) | pVCpu->iem.s.uRexReg);
-            IEM_MC_STORE_MEM_U256_ALIGN_AVX(pVCpu->iem.s.iEffSeg, GCPtrEffSrc, uSrc);
+            IEM_MC_FETCH_MEM_U256_ALIGN_AVX(uSrc, pVCpu->iem.s.iEffSeg, GCPtrEffSrc);
+            IEM_MC_STORE_YREG_U256_ZX_VLMAX(((bRm >> X86_MODRM_REG_SHIFT) & X86_MODRM_REG_SMASK) | pVCpu->iem.s.uRexReg, uSrc);
 
             IEM_MC_ADVANCE_RIP();
             IEM_MC_END();
         }
         return VINF_SUCCESS;
     }
+
     /**
      * @opdone
-     * @opmnemonic  udvex660f382areg
+     * @opmnemonic  udvex660f382arg
      * @opcode      0x2a
      * @opcodesub   11 mr/reg
      * @oppfx       0x66
      * @opunused    immediate
      * @opcpuid     avx
      * @optest      ->
-     * @oponly
      */
     return IEMOP_RAISE_INVALID_OPCODE();
 
 }
 
+
 /** Opcode VEX.66.0F38 0x2b. */
 FNIEMOP_STUB(iemOp_vpackusdw_Vx_Hx_Wx);
 /** Opcode VEX.66.0F38 0x2c. */
