@@ -7,7 +7,7 @@ Test Manager Core - Web Server Abstraction Base Class.
 
 __copyright__ = \
 """
-Copyright (C) 2012-2016 Oracle Corporation
+Copyright (C) 2012-2017 Oracle Corporation
 
 This file is part of VirtualBox Open Source Edition (OSE), as
 available from http://www.virtualbox.org. This file is free software;
@@ -26,7 +26,7 @@ CDDL are applicable instead of those of the GPL.
 You may elect to license modified versions of this file under the
 terms and conditions of either the GPL or the CDDL or both.
 """
-__version__ = "$Revision: 109040 $"
+__version__ = "$Revision: 118412 $"
 
 
 # Standard python imports.
@@ -45,7 +45,7 @@ from testmanager.core.base              import TMExceptionBase;
 from testmanager.core.globalresource    import GlobalResourceLogic;
 from testmanager.core.testboxstatus     import TestBoxStatusData, TestBoxStatusLogic;
 from testmanager.core.testbox           import TestBoxData, TestBoxLogic;
-from testmanager.core.testresults       import TestResultLogic;
+from testmanager.core.testresults       import TestResultLogic, TestResultFileData;
 from testmanager.core.testset           import TestSetData, TestSetLogic;
 from testmanager.core.systemlog         import SystemLogData, SystemLogLogic;
 from testmanager.core.schedulerbase     import SchedulerBase;
@@ -371,9 +371,9 @@ class TestBoxController(object): # pylint: disable=R0903
         self._checkForUnknownParameters();
 
         # Null conversions for new parameters.
-        if len(sReport) == 0:
+        if not sReport:
             sReport = None;
-        if len(sCpuName) == 0:
+        if not sCpuName:
             sCpuName = None;
         if lCpuRevision <= 0:
             lCpuRevision = None;
@@ -656,7 +656,7 @@ class TestBoxController(object): # pylint: disable=R0903
         # Parameter validation.
         #
         sBody = self._getStringParam(constants.tbreq.LOG_PARAM_BODY, fStrip = False);
-        if len(sBody) == 0:
+        if not sBody:
             return self._resultResponse(constants.tbresp.STATUS_NACK);
         self._checkForUnknownParameters();
 
@@ -703,28 +703,7 @@ class TestBoxController(object): # pylint: disable=R0903
                           ]:
             raise TestBoxControllerException('Invalid MIME type "%s"' % (sMime,));
 
-        if sKind not in [ 'log/release/vm',
-                          'log/debug/vm',
-                          'log/release/svc',
-                          'log/debug/svc',
-                          'log/release/client',
-                          'log/debug/client',
-                          'log/installer',
-                          'log/uninstaller',
-                          'log/guest/kernel',
-                          'crash/report/vm',
-                          'crash/dump/vm',
-                          'crash/report/svc',
-                          'crash/dump/svc',
-                          'crash/report/client',
-                          'crash/dump/client',
-                          'info/collection',
-                          'info/vgatext',
-                          'misc/other',
-                          'screenshot/failure',
-                          'screenshot/success',
-                          #'screencapture/failure',
-                          ]:
+        if sKind not in TestResultFileData.kasKinds:
             raise TestBoxControllerException('Invalid kind "%s"' % (sKind,));
 
         if len(sDesc) > 256:
@@ -773,7 +752,7 @@ class TestBoxController(object): # pylint: disable=R0903
         #
         sXml = self._getStringParam(constants.tbreq.XML_RESULT_PARAM_BODY, fStrip = False);
         self._checkForUnknownParameters();
-        if len(sXml) == 0: # Used for link check by vboxinstaller.py on Windows.
+        if not sXml: # Used for link check by vboxinstaller.py on Windows.
             return self._resultResponse(constants.tbresp.STATUS_ACK);
 
         (oDb, oStatusData, _) = self._connectToDbAndValidateTb([TestBoxStatusData.ksTestBoxState_Testing,
