@@ -10,7 +10,7 @@ Test Manager - Test result failures.
 
 __copyright__ = \
 """
-Copyright (C) 2012-2016 Oracle Corporation
+Copyright (C) 2012-2017 Oracle Corporation
 
 This file is part of VirtualBox Open Source Edition (OSE), as
 available from http://www.virtualbox.org. This file is free software;
@@ -29,7 +29,7 @@ CDDL are applicable instead of those of the GPL.
 You may elect to license modified versions of this file under the
 terms and conditions of either the GPL or the CDDL or both.
 """
-__version__ = "$Revision: 109040 $"
+__version__ = "$Revision: 118412 $"
 # Standard python imports.
 import unittest;
 
@@ -352,7 +352,7 @@ class TestResultFailureLogic(ModelLogicBase): # pylint: disable=R0903
             aoEntries.append(ChangeLogEntry(aoNew[1], None, aoNew[0], tsExpire, aoNew[2], aoOld[2], aoChanges));
 
         # If we're at the end of the log, add the initial entry.
-        if len(aaoRows) <= cMaxRows and len(aaoRows) > 0:
+        if len(aaoRows) <= cMaxRows and aaoRows:
             aoNew    = aaoRows[-1];
             tsExpire = aaoRows[-1 - 1][0] if len(aaoRows) > 1 else aoNew[2].tsExpire;
             aoEntries.append(ChangeLogEntry(aoNew[1], None, aoNew[0], tsExpire, aoNew[2], None, []));
@@ -386,7 +386,7 @@ class TestResultFailureLogic(ModelLogicBase): # pylint: disable=R0903
         #
         assert isinstance(oData, TestResultFailureData);
         dErrors = oData.validateAndConvert(self._oDb, oData.ksValidateFor_AddForeignId);
-        if len(dErrors) > 0:
+        if dErrors:
             raise TMInvalidData('editEntry invalid input: %s' % (dErrors,));
 
         # Check if it exist first (we're adding, not editing, collisions not allowed).
@@ -415,7 +415,7 @@ class TestResultFailureLogic(ModelLogicBase): # pylint: disable=R0903
         #
         assert isinstance(oData, TestResultFailureData);
         dErrors = oData.validateAndConvert(self._oDb, oData.ksValidateFor_Edit);
-        if len(dErrors) > 0:
+        if dErrors:
             raise TMInvalidData('editEntry invalid input: %s' % (dErrors,));
 
         oOldData = self.getById(oData.idTestResult)
